@@ -39,7 +39,8 @@ from lightbulb import errors
 from lightbulb import events
 from lightbulb import help as help_
 from lightbulb import plugins
-from lightbulb import stringview
+from lightbulb.converters import _DefaultingConverter
+from lightbulb.converters import _GreedyConverter
 from lightbulb.utils import maybe_await
 
 _LOGGER = logging.getLogger("lightbulb")
@@ -473,15 +474,20 @@ class Bot(hikari.BotApp):
         Returns:
             Optional[ :obj:`str` ]: Name of the command that was removed.
         """
-        command = self._commands.pop(name)
+        command = self._commands.pop(name, None)
+
+        if command is None:
+            return None
+
         self.commands.remove(command)
-        if command is not None:
-            keys_to_remove = [command.name, *command._aliases]
-            keys_to_remove.remove(name)
-            for key in keys_to_remove:
-                self._commands.pop(key)
-            _LOGGER.debug("command removed: %s", command.name)
-        return command.name if command is not None else None
+
+        keys_to_remove = [command.name, *command._aliases]
+        keys_to_remove.remove(name)
+        for key in keys_to_remove:
+            self._commands.pop(key)
+        _LOGGER.debug("command removed: %s", command.name)
+
+        return command.name
 
     def remove_plugin(self, name: str) -> typing.Optional[str]:
         """
@@ -493,21 +499,25 @@ class Bot(hikari.BotApp):
         Returns:
             Optional[ :obj:`str` ]: Name of the plugin that was removed.
         """
-        plugin = self.plugins.pop(name)
+        plugin = self.plugins.pop(name, None)
+
+        if plugin is None:
+            return None
+
         plugin.plugin_remove()
 
-        if plugin is not None:
-            for k in plugin._commands.keys():
-                self.remove_command(k)
+        for k in plugin._commands.keys():
+            self.remove_command(k)
 
-            for event_type, listeners in plugin.listeners.items():
-                for listener in listeners:
-                    callback = listener.__get__(plugin, type(plugin))
-                    self.unsubscribe(listener.event_type, callback)
-                    _LOGGER.debug("listener removed: %s (%s)", callback.__name__, listener.event_type.__name__)
+        for event_type, listeners in plugin.listeners.items():
+            for listener in listeners:
+                callback = listener.__get__(plugin, type(plugin))
+                self.unsubscribe(listener.event_type, callback)
+                _LOGGER.debug("listener removed: %s (%s)", callback.__name__, listener.event_type.__name__)
 
-            _LOGGER.debug("plugin removed: %s", plugin.name)
-        return plugin.name if plugin is not None else None
+        _LOGGER.debug("plugin removed: %s", plugin.name)
+
+        return plugin.name
 
     def load_extension(self, extension: str) -> None:
         """
@@ -741,16 +751,17 @@ class Bot(hikari.BotApp):
             return command
         raise errors.CommandNotFound(invoked_with)
 
-    def resolve_args_for_command(
-        self, command: commands.Command, raw_arg_string: str
+    async def resolve_args_for_command(
+        self, context: context_.Context, command: commands.Command, raw_arg_string: str
     ) -> typing.Tuple[typing.List[str], typing.Dict[str, str]]:
         """
         Resolve the appropriate command arguments from an unparsed string
-        containing the raw command arguments.
+        containing the raw command arguments and attempt to convert them into
+        the appropriate types given the type hint.
 
         This method can be overridden if you wish to customise how arguments
-        are parsed for all the commands. If you override this then it is important that
-        it at least returns a tuple containing an empty list if no positional arguments
+        are parsed and converted for all the commands. If you override this then it is important
+        that it at least returns a tuple containing an empty list if no positional arguments
         were resolved, and an empty dict if no keyword arguments were resolved.
 
         If you override this method then you may find the :obj:`~.stringview.StringView`
@@ -759,6 +770,7 @@ class Bot(hikari.BotApp):
         and which are optional or required.
 
         Args:
+            context (:obj:`~.context.Context`): The invocation context for the command.
             command (:obj:`~.commands.Command`): The command to resolve the arguments for.
             raw_arg_string (:obj:`str`): String containing the raw, unparsed arguments.
 
@@ -771,34 +783,45 @@ class Bot(hikari.BotApp):
                 arguments were supplied by the user.
             :obj:`~.errors.NotEnoughArguments`: Not enough arguments were provided by the user to fill
                 all required argument fields.
+            :obj:`~.errors.ConverterFailure`: Argument value conversion failed.
         """
-        sv = stringview.StringView(raw_arg_string)
-        positional_args, remainder = sv.deconstruct_str(max_parse=command.arg_details.maximum_arguments)
-        if remainder and command.arg_details.kwarg_name is None and not command._allow_extra_arguments:
-            raise errors.TooManyArguments(command)
-        if (len(positional_args) + bool(remainder)) < command.arg_details.minimum_arguments:
-            missing_args = command.arg_details.get_missing_args([*positional_args, *([remainder] if remainder else [])])
-            raise errors.NotEnoughArguments(command, missing_args)
+        converters = command.arg_details.converters[:]
+        arg_names = command.arg_details.arguments[:]
+        args, kwargs = [], {}
 
-        if not remainder:
-            remainder = {}
-        if remainder and command.arg_details.kwarg_name is not None:
-            remainder = {command.arg_details.kwarg_name: remainder}
-        return positional_args, remainder
+        arg_string = raw_arg_string
+        while converters:
+            conv = converters.pop(0)
+            arg_name = arg_names.pop(0)
+
+            if not isinstance(conv, (_DefaultingConverter, _GreedyConverter)) and not arg_string:
+                raise errors.NotEnoughArguments(command, [arg_name, *arg_names])
+
+            try:
+                conv_out, arg_string = await conv.convert(context, arg_string)
+            except (ValueError, TypeError, errors.ConverterFailure):
+                raise errors.ConverterFailure(f"Converting failed for argument: {arg_name}")
+
+            if isinstance(conv_out, dict):
+                kwargs.update(conv_out)
+            elif isinstance(conv, _GreedyConverter) and conv.unpack:
+                args.extend(conv_out)
+            else:
+                args.append(conv_out)
+
+        if arg_string and not command._allow_extra_arguments:
+            raise errors.TooManyArguments(command)
+
+        return args, kwargs
 
     async def _invoke_command(
         self,
         command: commands.Command,
         context: context_.Context,
         args: typing.Sequence[str],
-        kwarg: typing.Mapping[str, str],
+        kwargs: typing.Mapping[str, str],
     ) -> None:
-        if kwarg and command.arg_details.kwarg_name:
-            await command.invoke(context, *args, **kwarg)
-        elif args:
-            await command.invoke(context, *args)
-        else:
-            await command.invoke(context)
+        await command.invoke(context, *args, **kwargs)
 
     async def process_commands_for_event(self, event: hikari.MessageCreateEvent) -> None:
         """
@@ -854,13 +877,17 @@ class Bot(hikari.BotApp):
             await before_invoke(context)
 
         try:
-            positional_args, keyword_arg = self.resolve_args_for_command(command, final_args)
-            await self._evaluate_checks(command, context)
+            positional_args, keyword_arg = await self.resolve_args_for_command(context, command, final_args)
+            if not await maybe_await(command._check_exempt_predicate, context):
+                await self._evaluate_checks(command, context)
+            else:
+                _LOGGER.debug("Checks bypassed for command: %s", context.message.content)
         except (
             errors.NotEnoughArguments,
             errors.TooManyArguments,
             errors.CheckFailure,
             errors.CommandSyntaxError,
+            errors.ConverterFailure,
         ) as ex:
             await self._dispatch_command_error_event_from_exception(ex, event.message, context, command)
             return
