@@ -18,7 +18,6 @@
 from __future__ import annotations
 
 __all__: typing.Final[typing.List[str]] = [
-    "ArgInfo",
     "SignatureInspector",
     "Command",
     "Group",
@@ -26,7 +25,6 @@ __all__: typing.Final[typing.List[str]] = [
     "group",
 ]
 
-import dataclasses
 import functools
 import inspect
 import logging
@@ -40,7 +38,12 @@ from lightbulb import converters
 from lightbulb import cooldowns
 from lightbulb import errors
 from lightbulb import events
-from lightbulb.utils import maybe_await
+from lightbulb.converters import _BaseConverter
+from lightbulb.converters import _ConsumeRestConverter
+from lightbulb.converters import _Converter
+from lightbulb.converters import _DefaultingConverter
+from lightbulb.converters import _GreedyConverter
+from lightbulb.converters import _UnionConverter
 
 if typing.TYPE_CHECKING:
     from lightbulb import plugins
@@ -48,6 +51,23 @@ if typing.TYPE_CHECKING:
 _LOGGER = logging.getLogger("lightbulb")
 
 _CommandT = typing.TypeVar("_CommandT", bound="Command")
+T = typing.TypeVar("T")
+_ConverterT = typing.Union[_BaseConverter[T], _BaseConverter[typing.List[T]]]
+
+_CONVERTER_CLASS_MAPPING = {
+    hikari.User: converters.user_converter,
+    hikari.Member: converters.member_converter,
+    hikari.TextChannel: converters.text_channel_converter,
+    hikari.GuildVoiceChannel: converters.guild_voice_channel_converter,
+    hikari.GuildCategory: converters.category_converter,
+    hikari.Role: converters.role_converter,
+    hikari.Emoji: converters.emoji_converter,
+    hikari.GuildPreview: converters.guild_converter,
+    hikari.Message: converters.message_converter,
+    hikari.Invite: converters.invite_converter,
+    hikari.Colour: converters.colour_converter,
+    hikari.Color: converters.colour_converter,
+}
 
 
 class _BoundCommandMarker:
@@ -73,19 +93,10 @@ def _bind_prototype(instance: typing.Any, command_template: _CommandT):
 
         async def invoke(self, context: context_.Context, *args: str, **kwargs: str) -> typing.Any:
             if self.cooldown_manager is not None:
-                self.cooldown_manager.add_cooldown(context)
+                await self.cooldown_manager.add_cooldown(context)
             # Add the start slice on to the length to offset the section of arg_details being extracted
-            new_args = await self._convert_args(context, args, list(self.arg_details.args.values())[2 : len(args) + 2])
 
-            if kwargs:
-                new_kwarg = (
-                    await self._convert_args(
-                        context, kwargs.values(), [self.arg_details.args[self.arg_details.kwarg_name]]
-                    )
-                )[0]
-                kwargs = {self.arg_details.kwarg_name: new_kwarg}
-
-            return await self._callback(instance, context, *new_args, **kwargs)
+            return await self._callback(instance, context, *args, **kwargs)
 
     prototype = BoundCommand()
 
@@ -110,24 +121,6 @@ def _bind_prototype(instance: typing.Any, command_template: _CommandT):
     return typing.cast(_CommandT, prototype)
 
 
-@dataclasses.dataclass
-class ArgInfo:
-    """
-    Dataclass representing information for a single command argument.
-    """
-
-    ignore: bool
-    """:obj:`True` if the argument is ``self`` or ``context`` else :obj:`False`."""
-    argtype: int
-    """The type of the argument. See :attr:`inspect.Parameter.kind` for possible types."""
-    annotation: typing.Any
-    """The type annotation of the argument."""
-    required: bool
-    """Whether or not the argument is required during invocation."""
-    default: typing.Any
-    """Default value for an argument."""
-
-
 class SignatureInspector:
     """
     Contains information about the arguments that a command takes when
@@ -140,49 +133,78 @@ class SignatureInspector:
     def __init__(self, command: Command) -> None:
         self.command = command
         self.has_self = isinstance(command, _BoundCommandMarker)
-        self.kwarg_name = None
-        self.args = {}
         signature = inspect.signature(command._callback)
-        for index, (name, arg) in enumerate(signature.parameters.items()):
-            self.args[name] = self.parse_arg(index, arg)
+        self.converters = self.parse_signature(signature)
+        self.arguments = [p.name for p in signature.parameters.values()]
+        self.arguments.pop(0)
+        if self.has_self:
+            self.arguments.pop(0)
 
-        self.number_positional_args = 0
-        for arg in self.args.values():
-            if (
-                arg.argtype == inspect.Parameter.POSITIONAL_OR_KEYWORD
-                or arg.argtype == inspect.Parameter.POSITIONAL_ONLY
-            ) and not arg.ignore:
-                self.number_positional_args += 1
+    def get_converter(self, annotation) -> _ConverterT:
+        """
+        Resolve the converter order for a given type annotation recursively.
 
-        self.minimum_arguments = sum(1 for a in self.args.values() if not a.ignore and a.required)
-        self.maximum_arguments = (
-            float("inf")
-            if any(a.kind == inspect.Parameter.VAR_POSITIONAL for a in signature.parameters.values())
-            else self.number_positional_args
-        )
-        self.has_var_positional = any(a.kind == inspect.Parameter.VAR_POSITIONAL for a in signature.parameters.values())
+        Args:
+            annotation: The parameter's type annotation.
 
-    def parse_arg(self, index: int, arg: inspect.Parameter) -> ArgInfo:
-        if index == 0:
-            ignore = True
-        elif index == 1 and self.has_self:
-            ignore = True
-        else:
-            ignore = False
+        Returns:
+            The top level converter for the parameter
+        """
+        annotation = _CONVERTER_CLASS_MAPPING.get(annotation, annotation)
+        if isinstance(annotation, str):
+            annotation = str
 
-        argtype = arg.kind
-        annotation = arg.annotation
-        required = arg.default is inspect.Parameter.empty
-        default = arg.default
+        origin = typing.get_origin(annotation)
+        args = typing.get_args(annotation)
 
-        if arg.kind == inspect.Parameter.KEYWORD_ONLY and self.kwarg_name is None:
-            self.kwarg_name = arg.name
+        if origin is typing.Union:
+            convs: typing.List[_ConverterT] = []
 
-        return ArgInfo(ignore, argtype, annotation, required, default)
+            for arg in args:
+                if arg is None.__class__:
+                    return _DefaultingConverter(_UnionConverter(*convs), None, raise_on_fail=False)
 
-    def get_missing_args(self, args: typing.List[str]) -> typing.List[str]:
-        required_command_args = [name for name, arg in self.args.items() if not arg.ignore and arg.required]
-        return required_command_args[len(args) :]
+                convs.append(self.get_converter(arg))
+
+            return _UnionConverter(*convs)
+
+        if origin is converters.Greedy:
+            return _GreedyConverter(self.get_converter(args[0]))
+
+        return _Converter(annotation if annotation is not inspect.Parameter.empty else str)
+
+    def parse_signature(self, signature: inspect.Signature):
+        """
+        Parse the command's callback signature into a list of the converters used
+        for each command argument.
+
+        Args:
+            signature (:obj:`inspect.Signature`): The signature of the command callback.
+
+        Returns:
+            List of converters for command arguments in the correct order.
+        """
+        arg_converters = []
+
+        for idx, param in enumerate(signature.parameters.values()):
+            if idx == 0 or (idx == 1 and self.has_self):
+                continue
+
+            converter: _BaseConverter = self.get_converter(param.annotation)
+
+            if param.kind is inspect.Parameter.KEYWORD_ONLY:
+                converter = _ConsumeRestConverter(converter, param.name)
+            elif param.kind is inspect.Parameter.VAR_POSITIONAL:
+                converter = _GreedyConverter(converter, unpack=True)
+
+            if param.default is not inspect.Parameter.empty:
+                if not isinstance(converter, _DefaultingConverter):
+                    converter = _DefaultingConverter(converter, param.default)
+                else:
+                    converter.default = param.default
+
+            arg_converters.append(converter)
+        return arg_converters
 
 
 class Command:
@@ -217,6 +239,7 @@ class Command:
         self._aliases = aliases
         self.hidden = hidden
         self._checks = []
+        self._check_exempt_predicate = lambda ctx: False
         self._raw_error_listener = None
         self._raw_before_invoke = None
         self._raw_after_invoke = None
@@ -347,7 +370,7 @@ class Command:
         return self._callback
 
     @property
-    def checks(self) -> typing.Iterable[typing.Callable[[context.Context], bool]]:
+    def checks(self) -> typing.Iterable[typing.Callable[[context_.Context], bool]]:
         """
         The command's checks.
 
@@ -399,8 +422,8 @@ class Command:
         """
 
         def decorate(
-            func: typing.Callable[[context.Context], typing.Coroutine[None, None, typing.Any]]
-        ) -> typing.Callable[[context.Context], typing.Coroutine[None, None, typing.Any]]:
+            func: typing.Callable[[context_.Context], typing.Coroutine[None, None, typing.Any]]
+        ) -> typing.Callable[[context_.Context], typing.Coroutine[None, None, typing.Any]]:
             self._raw_before_invoke = func
             return func
 
@@ -415,67 +438,13 @@ class Command:
         """
 
         def decorate(
-            func: typing.Callable[[context.Context], typing.Coroutine[None, None, typing.Any]]
-        ) -> typing.Callable[[context.Context], typing.Coroutine[None, None, typing.Any]]:
+            func: typing.Callable[[context_.Context], typing.Coroutine[None, None, typing.Any]]
+        ) -> typing.Callable[[context_.Context], typing.Coroutine[None, None, typing.Any]]:
             self._raw_after_invoke = func
             return func
 
         return decorate
 
-    @staticmethod
-    async def handle_types(arg: str, type: typing.Any):
-        """
-        A method which handles converting Union (and therefore Optional) into objects usable by command functions.
-
-        Args:
-            arg (:obj:`str`): The argument's value to be converted.
-            type (:obj:`typing.Any`): Any type for the converter.
-        """
-
-        if typing.get_origin(type) is typing.Union:
-            for typename in (types := typing.get_args(type)) :
-                try:
-                    if typename is not None:
-                        new_arg = await maybe_await(typename, arg)
-                    else:
-                        new_arg = typename(arg)
-                    return new_arg
-                except (ValueError, TypeError, errors.ConverterFailure):
-                    if typename == types[-1]:
-                        raise errors.ConverterFailure
-                    else:
-                        continue
-        else:
-            new_arg = await maybe_await(type, arg)
-            return new_arg
-
-    async def _convert_args(
-        self,
-        context: context_.Context,
-        args: typing.Sequence[str],
-        arg_details: typing.Sequence[ArgInfo],
-    ) -> typing.Sequence[typing.Any]:
-        new_args = []
-
-        for arg, details in zip(args, arg_details):
-            arg = converters.WrappedArg(arg, context)
-            if details.annotation is inspect.Parameter.empty or isinstance(details.annotation, str):
-                new_args.append(str(arg))
-                continue
-            try:
-                new_arg = await self.handle_types(arg, details.annotation)
-                new_args.append(new_arg)
-            except (errors.ConverterFailure, ValueError):
-                _LOGGER.error(
-                    "Failed converting %s with converter: %s",
-                    arg,
-                    getattr(details.annotation, "__name__", repr(details.annotation)),
-                )
-                raise errors.ConverterFailure(
-                    text=f"Failed converting {arg} with converter: {getattr(details.annotation, '__name__', repr(details.annotation))}"
-                )
-        return new_args
-
     async def invoke(self, context: context_.Context, *args: str, **kwargs: str) -> typing.Any:
         """
         Invoke the command with given args and kwargs. Cooldowns and converters will
@@ -490,21 +459,11 @@ class Command:
 
         """
         if self.cooldown_manager is not None:
-            self.cooldown_manager.add_cooldown(context)
-        # Add the start slice on to the length to offset the section of arg_details being extracted
-        arg_details = list(self.arg_details.args.values())[1 : len(args) + 1]
-        new_args = await self._convert_args(context, args[: len(arg_details)], arg_details)
-        new_args = [*new_args, *args[len(arg_details) :]]
+            await self.cooldown_manager.add_cooldown(context)
 
-        if kwargs:
-            new_kwarg = (
-                await self._convert_args(context, kwargs.values(), [self.arg_details.args[self.arg_details.kwarg_name]])
-            )[0]
-            kwargs = {self.arg_details.kwarg_name: new_kwarg}
+        return await self._callback(context, *args, **kwargs)
 
-        return await self._callback(context, *new_args, **kwargs)
-
-    def add_check(self, check_func: typing.Callable[[context.Context], typing.Coroutine[None, None, bool]]) -> None:
+    def add_check(self, check_func: typing.Callable[[context_.Context], typing.Coroutine[None, None, bool]]) -> None:
         """
         Add a check to an instance of :obj:`~.commands.Command` or a subclass. The check passed must
         be an awaitable function taking a single argument which will be an instance of :obj:`~.context.Context`.
@@ -547,7 +506,13 @@ class Command:
             :obj:`~.errors.CheckFailure`: If the command is not runnable in the context given.
         """
         for check in self._checks:
-            result = await check(context)
+            try:
+                result = await check(context)
+            except errors.MissingRequiredAttachment:
+                # Bypass attachment check because we still want the command to show for the help
+                # overview and it is unlikely that someone will call the help command with an attachment
+                # to the message.
+                result = True
             if result is False:
                 raise errors.CheckFailure(f"Check {check.__name__} failed for command {self.name}")
         return True
