@@ -16,6 +16,8 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with Lightbulb. If not, see <https://www.gnu.org/licenses/>.
 """
+.. _converters:
+
 Lightbulb will attempt to convert command arguments using the type or function specified by the type
 hints of each argument. By default, any types that can be converted to from a string are supported - eg int or
 float.
@@ -36,6 +38,24 @@ You can also write your own converters. A converter can be any callable and shou
 will be an instance of the :obj:`~lightbulb.converters.WrappedArg` class. The arg value and command invocation
 context can be accessed through this instance from the attributes ``data`` and ``context`` respectively.
 
+Hikari classes are also available as type hints in place of the lightbulb converters and will be internally
+converted into the necessary converter for the command to behave as expected. A list of all available classes
+along with the converters they 'replace' can be seen below:
+
+- :obj:`hikari.User` (:obj:`~.user_converter`)
+- :obj:`hikari.Member` (:obj:`~.member_converter`)
+- :obj:`hikari.TextChannel` (:obj:`~.text_channel_converter`)
+- :obj:`hikari.GuildVoiceChannel` (:obj:`~.guild_voice_channel_converter`)
+- :obj:`hikari.GuildCategory` (:obj:`~.category_converter`)
+- :obj:`hikari.Role` (:obj:`~.role_converter`)
+- :obj:`hikari.Emoji` (:obj:`~.emoji_converter`)
+- :obj:`hikari.GuildPreview` (:obj:`~.guild_converter`)
+- :obj:`hikari.Message` (:obj:`~.message_converter`)
+- :obj:`hikari.Invite` (:obj:`~.invite_converter`)
+- :obj:`hikari.Colour` (:obj:`~.colour_converter`)
+- :obj:`hikari.Color` (:obj:`~.color_converter`)
+
+
 .. warning:: For the supplied converters, some functionality will not be possible depending on the intents and/or
     cache settings of your bot application and object. If the bot does not have a cache then the converters can
     only work for arguments of ID or mention and **not** any form of name.
@@ -60,6 +80,7 @@ __all__: typing.Final[typing.List[str]] = [
     "invite_converter",
     "colour_converter",
     "color_converter",
+    "Greedy",
 ]
 
 import collections
@@ -71,9 +92,11 @@ import hikari
 
 from lightbulb import context as context_
 from lightbulb import errors
+from lightbulb import stringview
 from lightbulb import utils
 
 T = typing.TypeVar("T")
+T_co = typing.TypeVar("T_co", covariant=True)
 
 USER_MENTION_REGEX: typing.Final[typing.Pattern] = re.compile(r"<@!?(\d+)>")
 CHANNEL_MENTION_REGEX: typing.Final[typing.Pattern] = re.compile(r"<#(\d+)>")
@@ -418,6 +441,163 @@ async def color_converter(arg: WrappedArg) -> hikari.Color:
     return await colour_converter(arg)
 
 
+class Greedy(typing.Generic[T]):
+    """
+    A special converter that greedily consumes arguments until it either runs out of arguments, or a parser error
+    is encountered. Due to this behaviour, most input errors will be silently ignored.
+
+    Example:
+
+        .. code-block:: python
+
+            @bot.command()
+            async def foo(ctx, foo: Greedy[int]):
+                # if called with <p>foo 1 2 3 4 5
+                # then the arg foo would contain [1, 2, 3, 4, 5]
+                ...
+    """
+
+
+class _ConverterT(typing.Protocol[T_co]):
+    def __call__(self, arg: WrappedArg) -> typing.Union[T_co, typing.Coroutine[None, None, T_co]]:
+        ...
+
+
+class _BaseConverter(typing.Protocol[T_co]):
+    async def convert(self, context: context_.Context, arg_string: str, *, parse: bool) -> typing.Tuple[T_co, str]:
+        ...
+
+
+class _Converter:
+    __slots__ = ("conversion_func",)
+
+    def __init__(self, conversion_func: _ConverterT) -> None:
+        self.conversion_func = conversion_func
+
+    async def convert(self, context: context_.Context, arg_string: str, *, parse: bool = True) -> typing.Tuple[T, str]:
+        args, remainder = arg_string, ""
+        if parse:
+            sv = stringview.StringView(arg_string)
+            parsed, remainder = sv.deconstruct_str(max_parse=1)
+            args = " ".join(parsed)
+
+        converted_arg = await utils.maybe_await(self.conversion_func, WrappedArg(args, context))
+        return converted_arg, remainder
+
+
+class _UnionConverter:
+    __slots__ = ("converters",)
+
+    def __init__(self, *converters: _BaseConverter[T]) -> None:
+        self.converters = converters
+
+    async def convert(self, context: context_.Context, arg_string: str, *, parse: bool = False) -> typing.Tuple[T, str]:
+        sv = stringview.StringView(arg_string)
+        args, remainder = sv.deconstruct_str(max_parse=1)
+
+        for converter in self.converters:
+            try:
+                converted_arg = await converter.convert(context, " ".join(args), parse=False)
+                break
+            except Exception:
+                continue
+        else:
+            raise errors.ConverterFailure
+
+        return converted_arg[0], remainder
+
+
+class _GreedyConverter:
+    __slots__ = ("converter", "unpack")
+
+    def __init__(self, converter: _BaseConverter[T], unpack: bool = False) -> None:
+        self.converter = converter
+        self.unpack = unpack
+
+    async def convert(
+        self, context: context_.Context, arg_string: str, *, parse: bool = False
+    ) -> typing.Tuple[typing.List[T], str]:
+        prev = arg_string
+        sv = stringview.StringView(arg_string)
+        converted = []
+
+        while True:
+            args, remainder = sv.deconstruct_str(max_parse=1)
+            if not args:
+                break
+
+            try:
+                converted_arg = await self.converter.convert(context, " ".join(args), parse=False)
+                converted.append(converted_arg[0])
+                prev = remainder
+            except Exception:
+                break
+
+        return converted, prev
+
+
+class _DefaultingConverter:
+    __slots__ = ("converter", "_default", "raise_on_fail")
+
+    def __init__(self, converter: _BaseConverter[T], default: typing.Any, raise_on_fail: bool = True):
+        self.converter = converter
+        self.raise_on_fail = raise_on_fail
+        self._default = default
+        self.apply_default_recursively(self.converter, default)
+
+    @property
+    def default(self) -> typing.Any:
+        if isinstance(self.converter, _ConsumeRestConverter):
+            # can't do the ternary here as for some reason, mypy can't tell if converter has "param_name" attr
+            return {self.converter.param_name: self._default}
+
+        return self._default
+
+    @default.setter
+    def default(self, default: typing.Any) -> None:
+        self._default = default
+
+    @staticmethod
+    def apply_default_recursively(converter: typing.Optional[_BaseConverter], default: typing.Any) -> None:
+        if converter is None:
+            return None
+
+        if isinstance(converter, _DefaultingConverter):
+            converter._default = default
+
+        return _DefaultingConverter.apply_default_recursively(getattr(converter, "converter", None), default)
+
+    async def convert(
+        self, context: context_.Context, arg_string: str, *, parse: bool = False
+    ) -> typing.Tuple[typing.Union[dict, T], str]:
+        if not arg_string:
+            return self.default, ""
+
+        try:
+            converted_arg = await self.converter.convert(context, arg_string, parse=True)
+        except Exception:
+            if self.raise_on_fail:
+                raise
+
+            return self.default, arg_string
+
+        return converted_arg
+
+
+class _ConsumeRestConverter:
+    __slots__ = ("converter", "param_name")
+
+    def __init__(self, converter: _BaseConverter[T], param_name: str):
+        self.converter = converter
+        self.param_name = param_name
+
+    async def convert(
+        self, context: context_.Context, arg_string: str, *, parse: bool = False
+    ) -> typing.Tuple[typing.Dict[str, T], str]:
+        converted_arg = await self.converter.convert(context, arg_string, parse=False)
+        return {self.param_name: converted_arg[0]}, ""
+
+
 if typing.TYPE_CHECKING:
     user_converter = hikari.User
     member_converter = hikari.Member
